<?php
// Database handling class;

class pgDB {

	var $host; #database host
	var $user; #database username
	var $pass; #...
	var $db;	#our database to select
	var $conn; #internal connection identifier
	var $query; #query string
	var $DB_ERROR; #database error

	function pgDB($dbuser=null, $dbpass=null, $dbname=null, $dbhost=null, $dbport=null) {
	
		$this->user = !empty($dbuser) ? $dbuser : DBUSER;
		$this->pass = $dbpass ? $dbpass : DBPWD;
		$this->db = $dbname ? $dbname : DBNAME;
		
		$dbhost = $dbhost ? $dbhost : DBHOST;
		$this->host = $dbhost ? $dbhost : "localhost";

		$this->port = $dbport ? $dbport : DBPORT;
		
		$connStr = "dbname=".$this->db;
		if ($dbhost) {
			$connStr .= " host=".$this->host;
		}
		if (!empty($this->port)) $connStr.= " port=".$this->port;
		if (!empty($this->user)) $connStr.= " user=".$this->user;
		if (!empty($this->pass)) $connStr.= " password=".$this->pass;
	
		$this->conn = pg_Connect($connStr)
			or die("Database connection lost!");

		#pg_set_client_encoding($this->conn, 'Latin2');
	}
	
	function getResult($query, $result_type = PGSQL_ASSOC) {
		if (empty($query)) {
			$this->DB_ERROR = 'EMPTY_QUERY';
			return false;
		}
		
		return new dbResult($this, $query, $result_type);
	}

	/// Start transaction
	function Begin() {
		pg_Exec("BEGIN");
	}

	/// finish transaction
	function Commit() {
		pg_Exec("COMMIT");
	}

	/// get next sequence id
	function NextId($sequence) {
		$Result = $this->getResult("SELECT NEXTVAL('\"$sequence\"') AS id;");
		
		if ($id = $Result->getField('id')) {
			unset($Result);
			return $id;
		}
// 		else {
// 			/// sequence probably not exists yet, create it now
// 			$this->Exec("CREATE SEQUENCE $sequence;");
// 			$this->NextId($sequence);
// 		}
		
		return false;
	}
	
	function Delete($table,$key,$id) {
		$query = 'BEGIN; DELETE FROM "'.$table.'" WHERE "'.$key.'" = '.$id.'; SELECT SETVAL(\'"'.$table.'_Id_seq"\',(SELECT CASE max("Id") WHEN 0 THEN 1 ELSE max("Id") END FROM "'.$table.'")); COMMIT;';
		if ($Result = new dbResult($this, $query, PGSQL_ASSOC)) {
			$Result->freeResult();
			return true;
		} else {
			return false;
		}
	}
	
	function updateTable($table, $data, $key = null) {
		if (!empty($key) && array_key_exists($key, $data)) {
			
			$chgval = $data[$key];
			
			if ($chgval == 'n') {
				/// we're going to insert new record
				$qStr1='';
				$qStr2='';
				
				$data[$key] = $this->NextId($table.'_'.$key.'_seq');
				
				foreach($data as $k=>$v) {
					$qStr1.="\"$k\","; $qStr2.="'".htmlspecialchars($v)."',";
				}
				
				$qStr="INSERT INTO \"$table\" (".substr($qStr1,0,-1).') VALUES ('.substr($qStr2,0,-1).')';
				unset($qStr1,$qStr2);

				/// return new id
				$result = $data[$key];
			} else {
				/// update existing record
				unset($data[$key]);
				
				$qStr = '';
				
				foreach($data as $k=>$v) {
					$qStr.="\"$k\"='".htmlspecialchars($v)."',";
				}
				
				$qStr = "UPDATE \"$table\" SET ".substr($qStr,0,-1)." WHERE \"$key\" = '$chgval'";
				
				/// return ID of changed record
				$result = $chgval;
			}
		} else {
			/// insert new record without returning new id
			$qStr1='';
			$qStr2='';
			
			foreach($data as $k=>$v) {
				$qStr1 .= "\"$k\",";
				$qStr2 .= "'".htmlspecialchars($v)."',";
				
				$result=true;
			}
			
			$qStr = "INSERT INTO \"$table\" (".substr($qStr1,0,-1).') VALUES ('.substr($qStr2,0,-1).');';
			unset($qStr1,$qStr2);
		}

		$query = $qStr;
		unset($qStr);
		
		if ($this->getResult($query)) {
			unset($query);
			return $result;
		}
		/// return false by defualt
		return false;
	}
	
	function close() {
		return pg_close($this->conn);
	}

	function debugDb($str) {
			echo $str."<br />";
	}
	
	function getTableStruct($table) {
//		$query = "SELECT attname AS field_name FROM pg_attribute WHERE attrelid = (SELECT oid FROM pg_class WHERE relname='".$table."') AND attnum > 0";
		$query = "SELECT a.attname AS field_name, t.typname AS field_type FROM pg_attribute a, pg_class c, pg_type t WHERE c.relname = '".$table."' AND a.attnum > 0 AND a.attrelid = c.oid AND a.atttypid = t.oid	ORDER BY a.attnum";

		$Result = $this->getResult($query);
		unset($query);
		$struct = array();
		while($res = $Result->fetchRow()) {
//			$struct[] = $res['field_name'];
			$struct[$res['field_name']] = $res['field_type'];

		}
		unset($Result, $res);
	return $struct;
}

###End of class definiton
}

class dbResult {
	
	var $DB;
	var $Query;
	var $Result;
	var $ResultType;
	var $Position;
	var $CurrentRow;
	var $NumRows;
	var $AffectedRows;
		
	function dbResult(&$DB, $query, $result_type) {
		global $_DEBUG;

		$this->DB = &$DB;
		$this->Query = $query;
			
		if ($this->Result) {
			$this->freeResult();
		}
			
		if ((DEBUG == 'db') || (DEBUG == 'all')) {
			debug($this->Query);
		}

		$this->ResultType = $result_type;
		$this->Position = 0;

		if (!$this->Result = pg_Exec($this->DB->conn, $this->Query)) {
			$this->DB->DB_ERROR = @pg_errormessage($this->DB->conn); #pg_result_error($this->Result);
			system_error($this->DB->DB_ERROR, FATAL);
			return false;
		}

		$this->NumRows = pg_num_rows($this->Result);
		$this->AffectedRows = pg_affected_rows($this->Result);
			
		return $this;
	}
		
	function fetchRow() {
		if ($this->Position < $this->NumRows) {
			$row = pg_fetch_array($this->Result, $this->Position, $this->ResultType);
			$this->CurrentRow = $row;
			$this->Position++;
			unset($row);
			return $this->CurrentRow;
		} else {
			### There are no more rows to return
			$this->freeResult();
			return false;
		}
	}

	function getCurrentRow() {
		return $this->CurrentRow;
	}
		
	function getField($field) {
		if (empty($this->CurrentRow)) {
			$this->fetchRow();
		}
		if (!empty($this->CurrentRow[$field])) {
			return $this->CurrentRow[$field];
		}
		return false;
	}

	function getNumRows() {
		return $this->NumRows;
	}
		
	function getAffectedRows() {
		return $this->AffectedRows;
	}
		
	function freeResult() {
		if (!pg_free_result($this->Result)) {
			$this->DB->DB_ERROR = pg_result_error($this->DB->conn);
			return false;
		}
		$this->Position = 0;
		$this->Query = null;
		$this->CurrentRow = null;
		$this->NumRows = 0;
		$this->AffectedRows = 0;
		return true;
	}
}
?>